@@ -36,7 +36,10 @@ pub fn lex(prg: &str) -> Vec<Token> {
         }
 
         if in_group && group_char.unwrap() == tok {
-            group_count += 1;
+            let last = buf.chars().last().unwrap_or('\u{2192}');
+            if group_char.unwrap() != '{' || (last != '.' && last != ':') {
+                group_count += 1;
+            }
         }
 
         if in_string {
@@ -48,6 +51,7 @@ pub fn lex(prg: &str) -> Vec<Token> {
                 buf.push(tok);
             }
         } else if in_group {
+            let last = buf.chars().last().unwrap_or('\u{2192}');
             if (tok == ')' || tok == '→') && Some('(') == group_char {
                 if group_count > 0 {
                     group_count -= 1;
@@ -57,7 +61,11 @@ pub fn lex(prg: &str) -> Vec<Token> {
                     buf.clear();
                     in_group = false;
                 }
-            } else if (tok == '}' || tok == '→') && Some('{') == group_char {
+            } else if (tok == '}' || tok == '→')
+                && Some('{') == group_char
+                && last != '.'
+                && last != ':'
+            {
                 if group_count > 0 {
                     group_count -= 1;
                     buf.push(tok);
@@ -137,9 +145,10 @@ pub fn lex(prg: &str) -> Vec<Token> {
                         .iter()
                         .rposition(|m| matches!(m, Token::Operator(_, _)))
                         .unwrap();
-                    // The previous op has a lower precedence, shouldn't have `_` inserted after it yet
+                    // The previous op has a lower precedence or no right rank, shouldn't have `_` inserted after it yet
                     let used_rank = if OPTIONS.precedence.get(ident).unwrap()
                         < OPTIONS.precedence.get(&buf).unwrap()
+                        && stack_rank.1 > 0
                     {
                         rank.0 as usize
                     } else {
@@ -210,41 +219,30 @@ fn expr_to_postfix(tokens: &[Token]) -> Vec<Token> {
     let mut output = Vec::with_capacity(tokens.len());
 
     for tok in tokens {
-        if let Token::Operator(left, rank) = tok {
+        if let Token::Operator(right, rank) = tok {
             while !operators.is_empty() {
                 let op = operators.pop().unwrap();
-                if let Token::Operator(ref right, _) = op {
+                if let Token::Operator(ref left, left_rank) = op {
                     if OPTIONS.precedence.get(left).is_none()
                         || OPTIONS.precedence.get(right).is_none()
-                        || OPTIONS.precedence.get(left).unwrap()
-                            > OPTIONS.precedence.get(right).unwrap()
+                        || (OPTIONS.precedence.get(right).unwrap()
+                            > OPTIONS.precedence.get(left).unwrap()
+                            && left_rank.1 > 0)
                         || rank.0 == 0
                     {
                         operators.push(op);
                         break;
                     }
-                }
 
-                // If there are missing implied `_` from the program, insert them now. These will always be the last ones (so this won't lead to interpretation issues)
-                // The above will hold true
-                let total_rank = rank.0
-                    + rank.1
-                    + output
-                        .iter()
-                        .cloned()
-                        .filter(|n| matches!(n, Token::Operator(_, _)))
-                        .fold(0, |acc, op| {
-                            //  This will always work
-                            if let Token::Operator(_, rank) = op {
-                                acc + rank.0 + rank.1
-                            } else {
-                                unreachable!()
-                            }
-                        });
-                for _ in 0..(total_rank - output.len() as i32) {
-                    output.push(Token::Variable('_'.to_string()));
+                    // If there are missing implied `_` from the program, insert them now. These will always be the last ones (so this won't lead to interpretation issues)
+                    // The above will hold true
+                    let total_rank =
+                        crate::utils::sum_rank(left_rank.0 as i128 + left_rank.1 as i128, &output);
+                    for _ in 0..(total_rank - output.len() as i128) {
+                        output.push(Token::Variable('_'.to_string()));
+                    }
+                    output.push(op)
                 }
-                output.push(op)
             }
 
             operators.push(tok.clone());
@@ -257,7 +255,11 @@ fn expr_to_postfix(tokens: &[Token]) -> Vec<Token> {
     }
 
     while !operators.is_empty() {
-        if let Token::Operator(_, _) = operators.last().unwrap() {
+        if let Token::Operator(_, rank) = operators.last().unwrap() {
+            let total_rank = crate::utils::sum_rank(rank.0 as i128 + rank.1 as i128, &output);
+            for _ in 0..(total_rank - output.len() as i128) {
+                output.push(Token::Variable('_'.to_string()));
+            }
             output.push(operators.pop().unwrap());
         } else {
             // Discard if invalid
