@@ -71,7 +71,7 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
         }
 
         // [<left>, <right>]
-        ",:" => {
+        "<>" => {
             let left = parse_node(Rc::clone(&env), &left[0]);
             let right = parse_node(Rc::clone(&env), &right[0]);
             Dynamic::from([left, right])
@@ -213,11 +213,15 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
 
         // Base conversion of <left> based on <right>
         ";" => {
-            let ops = parse_node(Rc::clone(&env), &right[0]).to_string();
+            let ops = if env.borrow().vals.get(&right[0].to_string()).is_none() {
+                right[0].to_string()
+            } else {
+                parse_node(Rc::clone(&env), &right[0]).to_string()
+            };
             let chars = ops.trim().trim_matches('"').chars();
             let mut cur = parse_node(Rc::clone(&env), &left[0]).to_string();
-            if cur.matches(' ').count() > 0 {
-                cur = cur.split(' ').collect::<Vec<_>>().join("");
+            if cur.matches('\n').count() > 0 {
+                cur = cur.split('\n').collect::<Vec<_>>().join("");
             }
             let mut num = String::new();
 
@@ -302,17 +306,26 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
             }
 
             Dynamic::from(
-                pre.clone()
-                    .get(0)
-                    .unwrap()
-                    .clone()
-                    .enumerate()
-                    .map(|(i, _)| {
-                        pre.iter()
-                            .filter_map(|array| array.clone().nth(i))
-                            .collect()
-                    })
-                    .collect::<Vec<Vec<Dynamic>>>(),
+                vec![
+                    0;
+                    pre.iter()
+                        .cloned()
+                        .max_by(|l, r| l.clone().count().cmp(&r.clone().count()))
+                        .unwrap_or_else(|| Sequence::from_vec_dyn(
+                            &[],
+                            Node::String(String::new()),
+                            Some(0)
+                        ))
+                        .count()
+                ]
+                .iter()
+                .enumerate()
+                .map(|(i, _)| {
+                    pre.iter()
+                        .filter_map(|array| array.clone().nth(i))
+                        .collect()
+                })
+                .collect::<Vec<Vec<Dynamic>>>(),
             )
         }
 
@@ -538,8 +551,9 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
                 let seperator = constructed.trim().trim_matches('_');
                 let program = res
                     .iter()
-                    .map(|n| format!("{}", n))
-                    .collect::<Vec<String>>()
+                    .cloned()
+                    .map(|n| format!("{}", n.into_node()))
+                    .collect::<Vec<_>>()
                     .join(seperator);
 
                 // This looks like a Vec<Node>, but in reality it is a single Node (only one value)
@@ -930,6 +944,26 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
             Dynamic::from(groups)
         }
 
+        "??" => {
+            let val = parse_node(Rc::clone(&env), &left[0]);
+            let condition = parse_node(Rc::clone(&env), &right[0]).literal_bool();
+            let child_env = Rc::new(env.as_ref().clone());
+
+            if condition {
+                if let Node::Block(_, name) = &right[1] {
+                    child_env
+                        .borrow_mut()
+                        .define_var(name.as_ref().unwrap_or(&USCORE), val.clone())
+                } else {
+                    child_env.borrow_mut().define_var("_", val.clone());
+                }
+
+                parse_node(Rc::clone(&child_env), &right[1])
+            } else {
+                val
+            }
+        }
+
         // Bind <right> to each value in <left>
         "@" => {
             let seq = parse_node(Rc::clone(&env), &left[0])
@@ -1065,6 +1099,18 @@ pub fn parse_node(env: Env, node: &Node) -> Dynamic {
     }
 }
 
+macro_rules! def_builtins {
+    ($env:ident; $($($name:literal),*: $value:literal);*) => {
+        $(
+            $env.define([$($name),*], |e, val| {
+                let child = Rc::new(e.as_ref().clone());
+                child.borrow_mut().define_var("_", val);
+                parse_node(Rc::clone(&child), &crate::build_ast($value)[0])
+            });
+        )*
+    }
+}
+
 pub fn parse(ast: &[Node]) {
     let mut env = Environment::init();
 
@@ -1139,45 +1185,16 @@ pub fn parse(ast: &[Node]) {
         println!("{}", d);
         d
     });
-    env.define(["f", "fact"], |e, val| {
-        if val.clone().literal_num() < 0 {
-            panic!("Cannot take factorial of non-zero number");
-        }
-        let child = Rc::new(e.as_ref().clone());
-        child.borrow_mut().define_var("_", val);
-        parse_node(Rc::clone(&child), &crate::build_ast(r#"*\(~||[1])"#)[0])
-    });
-    env.define(["me", "mean"], |e, val| {
-        let child = Rc::new(e.as_ref().clone());
-        child.borrow_mut().define_var("_", val);
-        parse_node(Rc::clone(&child), &crate::build_ast(r#"(+\)/(#"#)[0])
-    });
-    env.define(["ma", "max"], |e, val| {
-        let child = Rc::new(e.as_ref().clone());
-        child.borrow_mut().define_var("_", val);
-        parse_node(Rc::clone(&child), &crate::build_ast(r#"(:>) :{"#)[0])
-    });
-    env.define(["mi", "min"], |e, val| {
-        let child = Rc::new(e.as_ref().clone());
-        child.borrow_mut().define_var("_", val);
-        parse_node(Rc::clone(&child), &crate::build_ast(r#"(:<) :{"#)[0])
-    });
-    env.define(["med", "median"], |e, val| {
-        let child = Rc::new(e.as_ref().clone());
-        child.borrow_mut().define_var("_", val);
-        parse_node(
-            Rc::clone(&child),
-            &crate::build_ast(r#"(:-#&%2=0)&&:-(((:<)?(--:-#))+((:<)?:-#))||(:<)?:v:-#"#)[0],
-        )
-    });
-    env.define(["sdev"], |e, val| {
-        let child = Rc::new(e.as_ref().clone());
-        child.borrow_mut().define_var("_", val);
-        parse_node(
-            Rc::clone(&child),
-            &crate::build_ast(r#":/((@v{:*(v-me)).me"#)[0],
-        )
-    });
+    def_builtins! {env;
+        "f", "fact":        r#"*\(~||[1])"#;
+        "me", "mean":       r#"(+\)/(#"#;
+        "ma", "max":        r#"(:>) :{"#;
+        "mi", "min":        r#"(:<) :{"#;
+        "med", "median":    r#"(:-#&%2=0)&&:-(((:<)?(--:-#))+((:<)?:-#))||(:<)?:v:-#"#;
+        "sdev":             r#":/((@v{:*(v-me)).me"#;
+        "crt", "cartesian": r#":{@a{:}@a<>}&:_"#;
+        "eq", "equal":      r#":@#=1"#
+    };
 
     let env: Env = Rc::new(RefCell::new(env));
 
