@@ -72,7 +72,7 @@ macro_rules! hashmap {
 // First number is precedence, second is left # of args, third is right # of args
 operators! {
     '.': 11; 1-1,
-    '^': 10; 1-1, ",:": 10; 1-1,
+    '^': 10; 1-1, "<>": 10; 1-1,
     '*': 9; 1-1, '/': 9; 1-1,
     '%': 8; 1-1,
     ":|": 7; 1-1, ":!": 7; 1-1,
@@ -85,7 +85,7 @@ operators! {
     '|': 3; 1-1,
     '=': 2; 1-1, "!=": 2; 1-1, '<': 2; 1-1, "<=": 2; 1-1, '>': 2; 1-1, ">=": 2; 1-1,
     "&&": 1; 1-1, "||": 1; 1-1,
-    ':': 0; 1-1, "::": 0; 1-1, '@': 0; 1-1, '&': 0; 1-1, '$': 0; 0-2, "$:": 0; 0-2, "/:": 0; 0-2,
+    ':': 0; 1-1, "::": 0; 1-1, "??": 0; 1-2, '@': 0; 1-1, '&': 0; 1-1, '$': 0; 0-2, "$:": 0; 0-2, "/:": 0; 0-2,
     ":=": -1; 1-1
 }
 
